unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, CH341DLL;

type
  TForm1 = class(TForm)
    ChipIndex: TSpinEdit;
    OpenBtn: TButton;
    CloseBtn: TButton;
    RampBtn: TButton;
    Label1: TLabel;
    ReportList: TListBox;
    Memo1: TMemo;
    procedure OpenBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure RampBtnClick(Sender: TObject);
    procedure AddReport(aLine: String);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.OpenBtnClick(Sender: TObject);
begin
   //Verbindung zum Chip herstellen... ChipIndex: 0= 1.Chip am USB, 1= 2.Chip, usw.
   if (CH341OpenDevice(ChipIndex.Value)>0) then AddReport('DeviceOpen...OK')
                                           else AddReport('DeviceOpen...error!');
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
   //Verbindung schliessen...
   CH341CloseDevice(ChipIndex.Value);
   AddReport('Closed');
end;

procedure TForm1.RampBtnClick(Sender: TObject);
var outBytes: array[0..40] of Byte;
    i: word;
    aWord: Word;
    DATA, CLK, LOAD: Byte;

begin
   // LTC 1257 DA CONVERTER; 12 Bit seriell shiften; Laden mit /Load=LOW im 12. Takt
   // Signale selbst erzeugen, da LTC1257 nicht SPI konform
   //
   // LTC1257 pin      CH341A pin
   // ---------------------------
   // DATA                 D5
   // CLK                  D4
   // /LOAD                D2

   AddReport('Ramp 0...Vref with CH341BitStreamSPI');

   CH341Set_D5_D0(ChipIndex.Value,$3F,1); //D0...D5 auf Ausgang!

   For aWord:=0 to $FFF do //Rampe 0...Vref
   begin

      For i:=0 to 11 do // 12-Bi-Signalmuster erzeugen und in Outbytes ablegen...
      begin

         // Signale den Bitpositionen zuordnen;
         // DATA Signal (= D5; ; MSB first)
         // CLK Signal  (= D4)
         // LOAD Signal (= D2)
         if (aWord and (1 shl (11-i)))<>0 then DATA:=(1 shl 5) else DATA:=0;
         CLK:=(1 shl 4);
         LOAD:=(1 shl 2);

         // Muster fr ein Bit in jeweils 3 Bytes ablegen:
         // Erst DATA ausgeben und 1x takten, im 12. Takt /LOAD=0

         outBytes[i*3]:=LOAD or DATA    ;                //TAKT=LOW ; LOAD=HIGH; DATA halten
         outBytes[i*3+1]:=CLK or LOAD or DATA;           //TAKT HIGH; LOAD=HIGH; DATA halten
         if i<>11 then outBytes[i*3+2]:=LOAD or DATA     //TAKT LOW ; LOAD=HIGH; DATA halten
                  else outBytes[i*3+2]:=DATA;            //TAKT LOW ; LOAD LOW im 12 Takt; DATA halten
      end;
      OutBytes[36]:= LOAD or DATA;                       //TAKT LOW ; LOAD HIGH; DATA halten

      //Signalmuster (37 Bytes = Ein 12-Bit DA-Wert) ausgeben
      CH341BitStreamSPI(ChipIndex.Value,37,@outBytes);
   end;

   AddReport('Done.');

end;

procedure TForm1.AddReport(aLine: String);
begin
   ReportList.items.Add('Device '+IntToStr(ChipIndex.Value)+': '+aLine);
   if ReportList.items.Count>10 then ReportList.items.Delete(0);
   ReportList.Update;
end;

end.
