unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, CH341DLL;

// ***************************************************************
// LCD Text Display (HD44780 controller or compatible) with CH341A
//
// API: MemWriteAddr0, MemWriteAddr1
//
// Connections:
//
// LCD pin                   CH341A pin    comment
// ---------------------------------------------------------------
// D0...D7                    D0..D7   8 bit interface
// E                          /WR      clock inverted; no problem!
// RS                         A0       0=control reg. 1=data reg.
// R/W                        Ground   always write; never read
// ***************************************************************

Const

   //HD44780 instructions, compare HD44780 data sheet

   cmd_Clear_Display            = $01;
   cmd_Return_Home              = $02;
   cmd_Entry_Mode_Set           = $04;
   cmd_Display_On_Off           = $08;
   cmd_Cursor_or_Display_Shift  = $10;
   cmd_Function_Set             = $20;
   cmd_Set_CGRAM_Address        = $40;
   cmd_Set_DDRAM_Address        = $80;

   //HD44780 instruction params

   // cmd_Entry_Mode_Set params
   cursor_Shift_Left            =$00;
   cursor_Shift_Right           =$02;
   cursor_Shift_On              =$01;
   cursor_Shift_Off             =$00;

   // cmd_Display_On_Off params
   display_On                   =$04;
   display_Off                  =$00;
   cursor_On                    =$02;
   cursor_Off                   =$00;
   Cursor_Blink_On              =$01;
   Cursor_Blink_Off             =$00;

   // cmd_Cursor_or_Display_Shift params
   Shift_Display                =$08;
   Shift_Cursor                 =$00;
   Shift_Left                   =$04;
   Shift_Right                  =$00;

   //cmd_Function_Set params
   interface_8Bit               =$10;
   interface_4Bit               =$00;
   display_2lines               =$08;
   display_1line                =$00;
   Font_5x11                    =$04;
   Font_5x8                     =$00;


type

  TForm1 = class(TForm)
    ChipIndex: TSpinEdit;
    OpenBtn: TButton;
    CloseBtn: TButton;
    InitBtn: TButton;
    ClearBtn: TButton;
    Label1: TLabel;
    ReportList: TListBox;
    Memo1: TMemo;
    procedure OpenBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure InitBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure AddReport(aLine: String);
  private
    { Private-Deklarationen }
    procedure LCDInit;
    procedure WriteToLCDControlReg(aByte: Byte);
    procedure WriteToLCDDataReg(Data: PChar);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.OpenBtnClick(Sender: TObject);
begin
   //Verbindung zum Chip herstellen... ChipIndex: 0= 1.Chip am USB, 1= 2.Chip, usw.
   if (CH341OpenDevice(ChipIndex.Value)>0) then AddReport('DeviceOpen...OK')
                                           else AddReport('DeviceOpen...error!');
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
   //Verbindung schliessen...
   CH341CloseDevice(ChipIndex.Value);
   AddReport('Closed');
end;

procedure TForm1.WriteToLCDControlReg(aByte: Byte);
var BufferLen: cardinal;
begin
   BufferLen:=1;
   // Write with A0=0 selects LCD control reg.
   CH341MemWriteAddr0(ChipIndex.Value, @aByte, @BufferLen); //Adr. 0 = Control Reg.
   sleep(5); //Wait 5 ms for controller to execute command
end;

procedure TForm1.WriteToLCDDataReg(Data: PChar);
var BufferLen: cardinal;
    i: integer;
    aChar: Char;
begin
   BufferLen:=1;
   For i:=0 to strLen(Data)-1 do //write one by one; LCD needs delays!
   begin
      aChar:=Data[i];
      // Write with A0=1 selects LCD data reg.
      CH341MemWriteAddr1(ChipIndex.Value, @aChar, @BufferLen); //Adr. 1 = Data Reg.
      sleep(5); //Wait 5 ms for controller to execute command
   end;
end;

procedure TForm1.LCDInit;
var i: Integer;
begin
   WriteToLCDControlReg(cmd_Function_Set or interface_8Bit);
   sleep(10);
   For i:=1 to 3 do
   begin
      WriteToLCDControlReg(cmd_Function_Set or interface_8Bit);
   end;
   WriteToLCDControlReg(cmd_Function_Set or Interface_8Bit or display_2lines or Font_5x8);
   WriteToLCDControlReg(cmd_Display_On_Off or display_off or cursor_off or cursor_blink_off);
   WriteToLCDControlReg(cmd_Clear_Display);
   WriteToLCDControlReg(cmd_Entry_Mode_Set or cursor_Shift_Off or Cursor_Shift_right);
   WriteToLCDControlReg(cmd_Display_On_Off or display_On or cursor_off or cursor_blink_off);
end;

procedure TForm1.InitBtnClick(Sender: TObject);
var i: integer;
begin
   LCDInit;
   WriteToLCDDataReg('Hello world!');
   addReport('Hello world!');
end;

procedure TForm1.ClearBtnClick(Sender: TObject);
begin
   WriteToLCDControlReg(cmd_Clear_Display);
   addReport('Display cleared.');
end;

procedure TForm1.AddReport(aLine: String);
begin
   ReportList.items.Add('Device '+IntToStr(ChipIndex.Value)+': '+aLine);
   if ReportList.items.Count>10 then ReportList.items.Delete(0);
end;

end.
