unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, CH341DLL;

type
  TForm1 = class(TForm)
    ChipIndex: TSpinEdit;
    OpenBtn: TButton;
    CloseBtn: TButton;
    TempBtn: TButton;
    Label1: TLabel;
    ReportList: TListBox;
    Memo1: TMemo;
    WriteIOBtn: TButton;
    ReadIOBtn: TButton;
    ReadADCBtn: TButton;
    SweepDACBtn: TButton;
    WriteEEPROMBtn: TButton;
    ReadEEPROMBtn: TButton;
    procedure OpenBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure WriteIOBtnClick(Sender: TObject);
    procedure AddReport(aLine: String);
    procedure TempBtnClick(Sender: TObject);
    procedure ReadIOBtnClick(Sender: TObject);
    procedure ReadADCBtnClick(Sender: TObject);
    procedure SweepDACBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure WriteEEPROMBtnClick(Sender: TObject);
    procedure ReadEEPROMBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    LEDstatus: Byte;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.OpenBtnClick(Sender: TObject);
begin
   //Verbindung zum Chip herstellen... ChipIndex: 0= 1.Chip am USB, 1= 2.Chip, usw.
   if (CH341OpenDevice(ChipIndex.Value)>0) then AddReport('DeviceOpen...OK')
                                           else AddReport('DeviceOpen...error!');
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
   //Verbindung schliessen...
   CH341CloseDevice(ChipIndex.Value);
   AddReport('Closed');
end;

procedure TForm1.AddReport(aLine: String);
begin
   ReportList.items.Add(aLine);
   if ReportList.items.Count>10 then ReportList.items.Delete(0);
   ReportList.Update;
end;


procedure TForm1.TempBtnClick(Sender: TObject);
var inBytes, outBytes: array[0..10] of Byte;
     wLen, rLen: Cardinal;
     PTemp: PSmallInt;
begin
   // MCP9801 Temperature Sensor

   //Konfig Temp. Sensor...
   OutBytes[0]:=$92;                   // Slave Adress Temp. Sensor
   OutBytes[1]:=$01;                   // Configuration Register
   OutBytes[2]:=(1 shl 5);             // $20 = 10 bit resolution gives 1/4 C readings
   wLen:=3;                            // 1 Byte Slave Adr. + 1 Byte Register Adr. + 1 Byte Configuration = 3 Bytes
   rLen:=0;                            // nothing to read

   CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inbytes); //write: Slave Adr. + Reg.Adr. + Config.

   // also possible:
   // CH341WriteI2C(ChipIndex.Value, $92 shr 1, $01, 1 shl 5); // slave adress 0...127 in this case!


   //Temperature reading...
   OutBytes[0]:=$92; // Slave Adress Temp. Sensor
   OutBytes[1]:=$00; // Ambient Temperature Register
   wLen:=2;          // 1 Byte Slave Adr. + 1 Byte Register Adr. = 2 Bytes
   rLen:=2;          // Temp. Reg. is 16 Bit = 2 Bytes

   CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inbytes); //write: Slave Adr. + Reg.Adr.; read: 16 bit Temp.Reg

   // Calculate temperature from 2 Bytes read...
   PTemp:=@Inbytes[0];                          // 2 Byte make a SmallInt
   PTemp^:=swap(PTemp^);                        // Toggle Hi/Lo Byte
   AddReport(FloatTostr(PTemp^/256)+'C' );     // Temp.Integer: 1 LSB (virtual) = 1/256C

end;

procedure TForm1.WriteIOBtnClick(Sender: TObject);
var inBytes, outBytes: array[0..10] of Byte;
    wLen, rLen: Cardinal;
    PTemp: PSmallInt;
begin
   // MCP23008 8 Bit I/O-Expander

   //Konfig all GPIO as OUTPUT
   OutBytes[0]:=$40;                   // Slave Adress I/O Expander
   OutBytes[1]:=$00;                   // IODIR Register
   OutBytes[2]:=$00;                   // Port Direction: OUTPUT
   wLen:=3;                            // 1 Byte Slave Adr. + 1 Byte Register Adr. + 1 Byte Port Direction = 3 Bytes
   rLen:=0;                            // nothing to read

   CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inbytes); //write: Slave Adr. + Reg.Adr. + Port Dir.

   if LedStatus=0 then LedStatus:=1 else LedStatus:=LedStatus shl 1; //LED shift

   //Write GPIO Port Data
   OutBytes[0]:=$40;                   // Slave Adress I/O Expander
   OutBytes[1]:=$09;                   // GPIO Register
   OutBytes[2]:=LEDStatus;             // Port Data
   wLen:=3;                            // 1 Byte Slave Adr. + 1 Byte Register Adr. + 1 Byte Port Data = 3 Bytes
   rLen:=0;                            // nothing to read

   CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inbytes); //write: Slave Adr. + Reg.Adr. + Port Data.

   // also possible:
   // CH341WriteI2C(ChipIndex.Value,$40 shr 1,$09,LedStatus); // slave adress 0...127 in this case!

   AddReport('Port Write = $'+intToHex(LedStatus,2)); // Display Byte read

end;

procedure TForm1.ReadIOBtnClick(Sender: TObject);
var inBytes, outBytes: array[0..10] of Byte;
    wLen, rLen: Cardinal;
    PTemp: PSmallInt;
begin
   // MCP23008 8 Bit I/O-Expander

   //Read GPIO Port Status
   OutBytes[0]:=$40;                   // Slave Adress I/O Expander
   OutBytes[1]:=$09;                   // GPIO Register
   wLen:=2;                            // 1 Byte Slave Adr. + 1 Byte Register Adr. = 2 Bytes
   rLen:=1;                            // Read GPIO Register

   CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inBytes); //write: Slave Adr. + Reg.Adr.; read: GPIO Reg.

   AddReport('Port Read = $'+intToHex(inBytes[0],2)); // Display Byte read

end;

procedure TForm1.ReadADCBtnClick(Sender: TObject);

Const VDD=5.0;//[Volt]  (measured VDD)

var inBytes, outBytes: array[0..10] of Byte;
    wLen, rLen: Cardinal;
    ADCValue: Word;
begin
   // MCP3221 12 bit ADC

   //Read ADC
   OutBytes[0]:=$9A;                   // Slave Adress ADC
   wLen:=1;                            // 1 Byte Slave Adr.
   rLen:=2;                            // Read ADC Status = 2 Bytes

   CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inBytes); //write: Slave Adr.; read: ADC Status

   ADCValue:=inBytes[0]*256+inBytes[1]; //12 bit ADC value

   // Voltage value depending on VDD!
   AddReport('ADC Read = $'+intToHex(ADCValue,3)+' = '+FloatToStrF(ADCValue*VDD/$FFF,ffFixed,12,4)+' Volt');

end;

procedure TForm1.SweepDACBtnClick(Sender: TObject);
var inBytes, outBytes: array[0..10] of Byte;
    wLen, rLen: Cardinal;
    i, DacValue: Word;
begin
   // TC1321 10 bit DAC
   AddReport('Sweeping DAC 0...100%');

   For i:=0 to $3FF do //Sweep 10-Bit DAC value
   begin
      DACValue:=i shl 6;                  // Make 10 Bits left alligned!

      OutBytes[0]:=$90;                   // Slave Adress DAC
      OutBytes[1]:=$00;                   // Reg. Adr. DATA
      OutBytes[2]:=HiByte(DACValue);      // HiByte first
      OutBytes[3]:=LoByte(DACValue);      // LoByte second

      wLen:=4;                            // Slave Adr. + Reg.Adr. + 10 Bit DAC Value = 4 Bytes to write
      rLen:=0;                            // Nothing to read
      CH341StreamI2C(ChipIndex.Value,wlen,@outbytes,rlen,@inBytes); //write: Slave Adr. +Reg. Adr. + DAC value
   end;

   AddReport('Done.');
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   AddReport('Device is closed. Click OPEN first!');
end;

procedure TForm1.WriteEEPROMBtnClick(Sender: TObject);
var outBytes: array[0..10] of Byte;
    i: Word;
    aString: String;
begin
   For i:=0 to 7 do
   begin
      outBytes[i]:=random(256); //Generate random contents
      aString:=aString+intToHex(outBytes[i],2)+' ';
   end;
   AddReport('Write RANDOM values: '+aString);
   CH341WriteEEPROM(ChipIndex.Value, ID_24C02, $00, $08, @outBytes); //Write 8 Bytes to start adr. $00
end;

procedure TForm1.ReadEEPROMBtnClick(Sender: TObject);
var inBytes: array[0..10] of Byte;
    i: Word;
    aString: String;
begin
   CH341ReadEEPROM(ChipIndex.Value, ID_24C02, $00, $08, @inBytes); //Read 8 Bytes from start adr. $00
   For i:=0 to 7 do aString:=aString+intToHex(inBytes[i],2)+' ';
   AddReport('Read EEPROM values: '+aString);
end;

end.
